// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _API_H_
#define _API_H_

#include <cstddef>
#include <string>

#include "utils/service.h"

#include "config.h"
#include "message/container.h"
#include "statistics.h"

namespace zmq {
extern "C" {
#include <zmq.h>
}

class Message
{
public:
	Message(void);
	~Message(void);

	bool reserve(std::size_t _size);
	bool assign(const void *_data, std::size_t _size);
	bool assign(const std::string &_message);

	void *data(void);
	const void *data(void) const;
	std::size_t size(void) const;

	zmq_msg_t *message(void);

private:
	zmq_msg_t m_message;
};

class Socket
{
public:
	static const int RCVTIMEO = 200;

	Socket(void);
	~Socket(void);

	bool create(void *_context, int _type);
	void shutdown(void);
	bool running(void) const;

	bool setsockopt(int _name, const void *_value, std::size_t _len);

	bool bind(const std::string &_endpoint);
	bool connect(const std::string &_endpoint);

	bool send(Message &_message);
	bool recv(Message &_message);

private:
	void *m_socket;
};
}

class API : public service::Service
{
public:
	API(const Config &_config, const Statistics &_statistics);
	~API(void);

	virtual bool start(void);
	virtual void shutdown(void);
	virtual bool running(void) const;

	using downlink_type = message::downlink::Container;
	using uplink_type = message::uplink::Container;

	bool sendDownlink(const downlink_type &_message);

	bool recvUplink(uplink_type &_message);

#ifdef DEPRECATED_FEATURES
	bool requestStatistics(LoradStatistics &_statistics);
#endif

private:
	const Config &m_config;
	const Statistics &m_statistics;

	void *m_context;

	zmq::Socket m_control;
	zmq::Socket m_downlink;
	zmq::Socket m_uplink;
#ifdef DEPRECATED_FEATURES
	zmq::Socket m_lorad;
#endif

	bool subscribe(const char *_filter, std::size_t _size);
	bool subscribe(const std::string &_filter);

	bool send(zmq::Socket &_socket, const std::string &_message);
	bool recv(zmq::Socket &_socket, std::string &_message);

	bool rpc(void);
};

#endif
