// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include <chrono>
#include <cstddef>
#include <cstdint>
#include <ostream>
#include <string>
#include <vector>

#include "utils/strongtype.h"

struct Config
{
	using bool_type = utils::StrongType<bool, struct Bool>;
	using gwid_type = utils::StrongType<std::uint64_t, struct GatewayId>;

	gwid_type gatewayId;

	bool_type filterCRCValid;
	bool_type filterCRCInvalid;
	bool_type filterCRCNone;

	bool_type dbEnable;
	std::size_t dbLimitMessages;
	std::chrono::milliseconds dbDelayCooldown;

#ifdef DEPRECATED_FEATURES
	static const std::size_t GWMP_AUTORESTART_FALSE = 0;
	static const std::size_t GWMP_AUTORESTART_TRUE = 10;

	using gwmp_service_type =
		utils::StrongType<std::string, struct GWMPService>;
	using gwmp_autorestart_type =
		utils::StrongType<std::size_t, struct GWMPAutorestart>;

	std::string gwmpNode;
	gwmp_service_type gwmpServiceUplink;
	gwmp_service_type gwmpServiceDownlink;
	std::chrono::seconds gwmpPeriodHeartbeat;
	std::chrono::seconds gwmpPeriodStatistics;
	gwmp_autorestart_type gwmpAutorestart;
	std::size_t gwmpLimitDatagram;
	std::string gwmpLoradControl;
#endif

	std::vector<std::string> apiUplink;
	std::vector<std::string> apiDownlink;
	std::vector<std::string> apiControl;
	std::vector<std::string> apiFilters;

	Config(void);

	bool parse_file(const std::string &_file);
};

std::ostream &operator<<(std::ostream &_os, const Config &_config);

#endif
