// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _DATABASE_H_
#define _DATABASE_H_

#include <condition_variable>
#include <cstddef>
#include <cstdint>
#include <mutex>
#include <vector>

#include "third-party/sqlite3pp.h"

#include "utils/service.h"

#include "config.h"
#include "message/container.h"

class Database : public service::Service
{
public:
	Database(const Config &_config);

	virtual bool start(void);
	virtual void shutdown(void);

	using uplink_type = message::uplink::Container;
	using uplinks_type = std::vector<uplink_type>;
	using id_type = message::uint_type;
	using ids_type = std::vector<id_type>;

	bool insert(const uplink_type &_message);

	bool fetch(uplinks_type &_messages, std::size_t _size);

	bool erase(const ids_type &_ids);

	class SQLite
	{
	public:
		const int DBVERSION = 2;

		static const int MODULATION_LORA = 0;
		static const int MODULATION_GFSK = 1;

		SQLite(const std::string &_filename);

		bool open(void);

		bool fetch(uplinks_type &_messages, std::size_t _size);

		std::size_t size(void);

		bool clear(void);
		bool erase(id_type _id);
		bool erase(const ids_type &_ids);
		bool pop_back(void);
		bool push_front(const uplink_type &_message);

	private:
		bool create(void);
		bool update(int from);

		std::string m_filename;
		sqlite3pp::database m_db;
	};

private:
	const Config &m_config;

	SQLite m_sqlite;

	std::condition_variable m_not_empty;
	std::mutex m_mutex;
	std::size_t m_size;
};

#endif
