// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "filter.h"

#include "third-party/variant.h"

#include "utils/logger.h"
#include "utils/type_traits.h"

Filter::Filter(const Config &_config) : m_config(_config) {}

namespace {
struct UplinkVisitor
{
	const Config &m_config;

	UplinkVisitor(const Config &_config) : m_config(_config) {}

	bool operator()(...) { return true; }

	template <typename T,
		  typename = utils::enable_if_t<
			  utils::is_one_of<T,
					   message::lora::Uplink,
					   message::gfsk::Uplink>::value>>
	bool operator()(const T &_message)
	{
		switch (_message.data.crc) {
		case message::uplink::CRCStatus::CRC_OK:
			if (m_config.filterCRCValid) {
				break;
			}

			logr::info << "Dropping message with valid CRC"
				   << logr::endl;
			return false;

		case message::uplink::CRCStatus::CRC_ERROR:
			if (m_config.filterCRCInvalid) {
				break;
			}

			logr::info << "Dropping message with invalid CRC"
				   << logr::endl;
			return false;

		case message::uplink::CRCStatus::CRC_NONE:
			if (m_config.filterCRCNone) {
				break;
			}

			logr::info << "Dropping message without CRC"
				   << logr::endl;
			return false;
		}

		return true;
	}
};
}

bool Filter::input(const uplink_type &_message)
{
	return mpark::visit(UplinkVisitor(m_config), _message.messages);
}
