// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "message/common.h"

#include <algorithm>
#include <iomanip>
#include <sstream>

namespace message {
namespace time {
std::string to_string(std::time_t _time,
		      StringFormat _format,
		      uint_type _microseconds,
		      bool _has_microseconds)
{
	std::ostringstream oss;
	const char *format;
	char cstr[32];

	switch (_format) {
	case StringFormat::GWMP_EXPANDED:
		format = "%F %T";
		break;
	default:
		format = "%FT%T";
		break;
	}

	if (!std::strftime(cstr, sizeof(cstr), format, std::gmtime(&_time))) {
		return "unknown";
	}

	oss << cstr;

	if (_has_microseconds) {
		std::ostringstream oss_ms;

		// Add leading zeros.
		oss_ms << std::setfill('0') << std::setw(6)
		       << (_microseconds % 1000000);

		std::string str_ms = oss_ms.str();

		// Remove trailing zeros.
		str_ms.erase(std::find_if(str_ms.rbegin(),
					  str_ms.rend(),
					  [](int c) { return c != '0'; })
				     .base(),
			     str_ms.end());

		// Print something if null
		if (str_ms.empty()) {
			str_ms = "0";
		}

		oss << "." << str_ms;
	}

	switch (_format) {
	case StringFormat::GWMP_EXPANDED:
		oss << " GMT";
		break;
	default:
		oss << "Z";
		break;
	}

	return oss.str();
}

std::string to_string(const Time &_time, StringFormat _format)
{
	std::time_t time = _time.time;

	if (_time.format == TimeFormat::GPS_TIME) {
		time += GPS_UNIX_EPOCH;
	}

	return to_string(
		time, _format, _time.microseconds, _time.has_microseconds);
}

std::string to_string(const AccurateContinuousTime &_time, StringFormat _format)
{
	std::time_t time = _time.time;

	if (_time.format == ContinuousTimeFormat::GPS_TIME) {
		time += GPS_UNIX_EPOCH;
	}

	return to_string(time, _format, _time.microseconds, true);
}
}
}
