// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _MESSAGE_COMMON_H_
#define _MESSAGE_COMMON_H_

#include <cstdint>
#include <ctime>
#include <string>

namespace message {
using int_type = std::int32_t;
using uint_type = std::uint32_t;
using float_type = float;
using bool_type = bool;

namespace time {
// The offset between the GPS epoch and the UNIX epoch.
// This is also the UNIX time of 1980-01-06T00:00:00.
const int_type GPS_UNIX_EPOCH = 315964800;

// The offset between the TAI time and the GPS time.
// TAI time is always ahead of GPS time by 19 seconds.
const int_type TAI_GPS_OFFSET = 19;

enum class TimeFormat : uint_type {
	UNIX_TIME = 0,
	UNIX_EPOCH_UTC_TIME = 0,
	TAI_TIME = 1,
	UNIX_EPOCH_TAI_TIME = 1,
	GPS_TIME = 2,
	GPS_EPOCH_TAI_TIME = 2,
};

struct Time
{
	TimeFormat format;
	int_type time;
	bool_type has_microseconds;
	uint_type microseconds;
};

enum class ContinuousTimeFormat : uint_type {
	TAI_TIME = 0,
	UNIX_EPOCH_TAI_TIME = 0,
	GPS_TIME = 1,
	GPS_EPOCH_TAI_TIME = 1,
};

struct AccurateContinuousTime
{
	ContinuousTimeFormat format;
	int_type time;
	uint_type microseconds;
};

enum class StringFormat {
	ISO8601_EXTENDED,
	GWMP_COMPACT = ISO8601_EXTENDED,
	GWMP_EXPANDED,
};

std::string to_string(std::time_t _time,
		      StringFormat _format = StringFormat::ISO8601_EXTENDED,
		      uint_type _microseconds = 0,
		      bool _has_microseconds = false);
std::string to_string(const Time &_time,
		      StringFormat _format = StringFormat::ISO8601_EXTENDED);
std::string to_string(const AccurateContinuousTime &_time,
		      StringFormat _format = StringFormat::ISO8601_EXTENDED);
}
}

#endif
