// Copyright (c) 2018 Kerlink SA. All rights reserved.

#include "message/event.h"

#include "utils/stream.h"

namespace message {
namespace event {
namespace {
std::string to_string(Event _event)
{
	switch (_event) {
	case Event::CONNECTED:
		return "Connected";
	case Event::STOPPED:
		return "Stopped";
	case Event::READY:
		return "Ready";
	case Event::STARTED:
		return "Started";
	case Event::MONITORED:
		return "Monitored";
	case Event::SYNCHRONIZED:
		return "Synchronized";
	case Event::GPS_LOCKED:
		return "GPS locked";
	case Event::GPS_UNLOCKED:
		return "GPS unlocked";
	case Event::SCAN_STARTED:
		return "Scan started";
	case Event::SCAN_STOPPED:
		return "Scan stopped";
	default:
		return "Unknown";
	}
}

std::string to_string(Status _status)
{
	switch (_status) {
	case Status::TRANSMITTED:
		return "Transmitted";
	case Status::TRANSMISSION_FAILED:
		return "Transmission failed";
	case Status::NOT_REQUIRED:
		return "Not required";
	case Status::BAD_MESSAGE_FORMAT:
		return "Bad message format";
	case Status::BAD_TRANSMISSION_FORMAT:
		return "Bas transmission format";
	case Status::BAD_POWER:
		return "Bad power";
	case Status::BAD_FREQUENCY:
		return "Bad frequency";
	case Status::BAD_TIMING:
		return "Bad timing";
	case Status::GPS_UNLOCKED:
		return "GPS unlocked";
	case Status::SCHEDULING_FAILED:
		return "Scheduling failed";
	case Status::CHANNEL_BUSY:
		return "Channel busy";
	case Status::MAXIMUM_DUTY_CYCLE:
		return "Maximum duty cycle";
	case Status::NOT_SYNCHRONIZED:
		return "Not synchronized";
	default:
		return "Unknown";
	}
}
}

std::ostream &operator<<(std::ostream &_os, const System &_message)
{
	_os << "system event \"" << to_string(_message.event) << "\"";

	return _os;
}

std::ostream &operator<<(std::ostream &_os, const Transmission &_message)
{
	_os << "transmission event (" << utils::to_hex(_message.message_id)
	    << " / " << utils::to_hex(_message.transmission_id) << ")";

	_os << ", status \"" << to_string(_message.status) << "\"";

	if (_message.has_radio_id) {
		_os << ", radio (" << utils::to_hex(_message.radio_id) << ")";
	}

	if (_message.pre_delay) {
		_os << ", delay -" << _message.pre_delay << " usec";
	} else if (_message.post_delay) {
		_os << ", delay +" << _message.pre_delay << " usec";
	}

	if (_message.has_time) {
		_os << ", time " << to_string(_message.time);
	}

	if (_message.has_power) {
		_os << ", power " << _message.power << " dB";
	}

	return _os;
}

namespace {
struct EventVisitor
{
	std::ostream &m_os;

	EventVisitor(std::ostream &_os) : m_os{_os} {}

	template <typename T>
	void operator()(const T &_message)
	{
		m_os << _message;
	}
};
}

std::ostream &operator<<(std::ostream &_os, const Uplink &_message)
{
	mpark::visit(EventVisitor(_os), _message.events);

	return _os;
}
}
}
