// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _MESSAGE_EVENT_H_
#define _MESSAGE_EVENT_H_

#include <cstdint>
#include <ostream>

#include "third-party/variant.h"

#include "message/common.h"

namespace message {
namespace event {
enum class Event : uint_type {
	CONNECTED = 0,
	STOPPED = 1,
	READY = 2,
	STARTED = 3,
	MONITORED = 4,
	SYNCHRONIZED = 9,
	GPS_LOCKED = 10,
	GPS_UNLOCKED = 11,
	SCAN_STARTED = 12,
	SCAN_STOPPED = 13,
};

struct System
{
	Event event;
};

enum class Status : uint_type {
	TRANSMITTED = 0,
	TRANSMISSION_FAILED = 1,
	NOT_REQUIRED = 2,
	BAD_MESSAGE_FORMAT = 3,
	BAD_TRANSMISSION_FORMAT = 4,
	BAD_POWER = 5,
	BAD_FREQUENCY = 6,
	BAD_TIMING = 7,
	GPS_UNLOCKED = 8,
	SCHEDULING_FAILED = 9,
	CHANNEL_BUSY = 10,
	MAXIMUM_DUTY_CYCLE = 11,
	NOT_SYNCHRONIZED = 12,
};

struct Transmission
{
	uint_type message_id;
	uint_type transmission_id;
	Status status;
	bool_type has_radio_id;
	uint_type radio_id;
	uint_type pre_delay;
	uint_type post_delay;
	bool_type has_time;
	time::Time time;
	bool_type has_power;
	float_type power;
};

struct Uplink
{
	Uplink(void) {}
	Uplink(const System &_event) : events(_event) {}
	Uplink(const Transmission &_event) : events(_event) {}

	mpark::variant<System, Transmission> events;
};

std::ostream &operator<<(std::ostream &_os, const System &_message);
std::ostream &operator<<(std::ostream &_os, const Transmission &_message);
std::ostream &operator<<(std::ostream &_os, const Uplink &_message);
}
}

#endif
