// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _UTILS_DAEMON_H_
#define _UTILS_DAEMON_H_

#include <csignal>
#include <string>
#include <vector>

// This daemon module owns several utilities for SysV daemons:
// https://www.freedesktop.org/software/systemd/man/daemon.html
//
// 1. The daemonize() function will run the process in the background.
// 2. The handleSignals() function provide an easy way to configure signals.
// 3. The PidFile class handle a pid file following the RAII principle.

namespace daemon {
int daemonize(void);

struct signal
{
	int num;
	sighandler_t handler;
};

bool handleSignals(const std::vector<struct signal> &_signals);

class PidFile
{
public:
	PidFile(void);
	~PidFile(void);

	bool create(const char *_file);
	bool create(const std::string &_file);

private:
	std::string m_file;
	int m_fd;
};
}

#endif
