// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _UTILS_LORAWAN_H_
#define _UTILS_LORAWAN_H_

#include <cstddef>
#include <cstdint>
#include <ostream>
#include <string>

namespace utils {
struct LoRaWAN
{
	static const int MAJOR_VERSION = 0;

	LoRaWAN(const std::string &_payload);
	LoRaWAN(const std::uint8_t *_payload, std::size_t _size);

	// PHY payload
	const std::uint8_t *payload;
	std::size_t size;

	// Message Header
	enum MessageType : std::uint8_t {
		TYPE_JOIN_REQUEST = 0,
		TYPE_JOIN_ACCEPT = 1,
		TYPE_UNCONFIRMED_DATA_UP = 2,
		TYPE_UNCONFIRMED_DATA_DOWN = 3,
		TYPE_CONFIRMED_DATA_UP = 4,
		TYPE_CONFIRMED_DATA_DOWN = 5,
		TYPE_REJOIN_REQUEST = 6,
		TYPE_PROPRIETARY = 7,
		TYPE_UNKNOWN
	};

	enum MessageType messageType;

	bool isData(void) const;

	enum Direction { DIR_UPLINK, DIR_DOWNLINK, DIR_UNKNOWN };

	Direction direction;

	std::uint8_t majorVersion;

	// Data
	std::uint32_t deviceAddress;
	std::uint8_t frameControl;
	bool hasAdaptativeDataRate;
	bool hasADRAcknowlegdementRequest;
	bool hasAcknowlegdement;
	bool hasFramePending;
	bool hasClassB;
	std::uint8_t frameOptionsLength;
	std::uint16_t frameCounter;
	bool hasFramePort;
	std::uint8_t framePort;

	// Join request
	std::uint64_t joinEUI;
	std::uint64_t deviceEUI;
	std::uint16_t deviceNonce;

	// Rejoin request
	std::uint8_t rejoinType;
	std::uint32_t netID;
	std::uint16_t rejoinCount;
};

std::ostream &operator<<(std::ostream &_os, const LoRaWAN &_lorawan);
}

#endif
