// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _UTILS_STREAM_H_
#define _UTILS_STREAM_H_

#include <cstddef>
#include <iomanip>
#include <iterator>
#include <ostream>
#include <sstream>
#include <string>
#include <vector>

namespace utils {
// Helper to easily print an hexadecimal value.
template <typename T>
struct ToHex
{
	T value;
	std::size_t width;
};

template <typename T>
ToHex<T> to_hex(const T &_value, std::size_t _width = 0)
{
	std::size_t max = sizeof(T) << 1;
	return {_value, (_width) ? (_width < max) ? _width : max : max};
}

template <>
ToHex<std::string> to_hex(const std::string &_value, std::size_t _width);

template <typename T>
std::ostream &operator<<(std::ostream &_os, const ToHex<T> &_hex)
{
	std::ostringstream string;

	string << std::hex << std::setfill('0') << std::uppercase;
	string << std::setw(_hex.width)
	       << static_cast<std::uintmax_t>(_hex.value);

	return (_os << string.str());
}

template <>
std::ostream &operator<<(std::ostream &_os, const ToHex<char> &_hex);

template <>
std::ostream &operator<<(std::ostream &_os, const ToHex<std::string> &_hex);
}

namespace std {
// Helper to easily print a vector to a stream.
template <typename T>
ostream &operator<<(ostream &_os, const vector<T> &_values)
{
	_os << "[ ";

	for (auto value = _values.begin(); value != _values.end(); ++value) {
		_os << *value;

		if (next(value) != _values.end()) {
			_os << ", ";
		}
	}

	return (_os << " ]");
}
}

#endif
