// Copyright (c) 2018 Kerlink SA. All rights reserved.

#ifndef _UTILS_TYPE_TRAITS_H_
#define _UTILS_TYPE_TRAITS_H_

#include <cstdint>
#include <type_traits>

namespace utils {
template <bool B, class T, class F>
using conditional_t = typename std::conditional<B, T, F>::type;

template <class...>
struct disjunction : std::false_type
{
};

template <class B1>
struct disjunction<B1> : B1
{
};

template <class B1, class... Bn>
struct disjunction<B1, Bn...>
	: conditional_t<bool(B1::value), B1, disjunction<Bn...>>
{
};

template <class T>
using decay_t = typename std::decay<T>::type;

template <typename T, typename... Ts>
struct is_one_of : disjunction<std::is_same<decay_t<T>, Ts>...>
{
};

template <bool B, class T = void>
using enable_if_t = typename std::enable_if<B, T>::type;

template <typename T>
struct is_signed_int : is_one_of<typename std::remove_cv<T>::type,
				 std::int8_t,
				 std::int16_t,
				 std::int32_t,
				 std::int64_t>::type
{
};

template <typename T>
struct is_unsigned_int : is_one_of<typename std::remove_cv<T>::type,
				   std::uint8_t,
				   std::uint16_t,
				   std::uint32_t,
				   std::uint64_t>::type
{
};

template <typename T>
struct is_int : disjunction<is_signed_int<T>, is_unsigned_int<T>>
{
};

template <typename T>
struct is_float
	: is_one_of<typename std::remove_cv<T>::type, float, double>::type
{
};
}

#endif
